/*
 * PhoneGap is available under *either* the terms of the modified BSD license *or* the
 * MIT License (2008). See http://opensource.org/licenses/alphabetical for full text.
 *
 * Copyright 2012, Andrew Lunny, Adobe Systems
 * Copyright (c) 2005-2010, Nitobi Software Inc.
 * Copyright (c) 2011, IBM Corporation
 * (c) 2010 Jesse MacFadyen, Nitobi
 */

var ChildBrowser = (function (gap) {
	var ref;
	
    function isFunction(f) {
        return typeof f === "function";
    }

    // placeholder and constants
    function ChildBrowser() {}

    var CLOSE_EVENT = 0,
        LOCATION_CHANGED_EVENT = 1,
        OPEN_EXTERNAL_EVENT = 2;

    /**
     * Function called when the child browser has an event.
     */
    function onEvent(data) {
        switch (data.type) {
            case CLOSE_EVENT:
                if (isFunction(ChildBrowser.onClose)) {
                    ChildBrowser.onClose();
                }
                break;
            case LOCATION_CHANGED_EVENT:
                if (isFunction(ChildBrowser.onLocationChange)) {
                    ChildBrowser.onLocationChange(data.location);
                }
                break;
            case OPEN_EXTERNAL_EVENT:
                if (isFunction(ChildBrowser.onOpenExternal)) {
                    ChildBrowser.onOpenExternal();
                }
                break;
        }
    }

    /**
     * Function called when the child browser has an error.
     */
    function onError(data) {
        if (isFunction(ChildBrowser.onError)) {
            ChildBrowser.onError(data);
        }
    }

    /**
     * Maintain API consistency with iOS
     */
    ChildBrowser.install = function () {
        console.log('ChildBrowser.install is deprecated');
    }

    /**
     * Display a new browser with the specified URL.
     * This method loads up a new web view in a dialog.
     *
     * @param url           The url to load
     * @param options       An object that specifies additional options
     */
    ChildBrowser.showWebPage = function (url, options) {
        if (!options) {
            options = { showLocationBar: true };
        }
        ref = window.open(url, "_blank", "location=" + (options.showLocationBar && options.showAddress ? "yes" : "no") + ",toolbar=" + (options.showNavigationBar ? "yes" : "no"));
        return ref;
    };

    /**
     * Close the browser opened by showWebPage.
     */
    ChildBrowser.close = function () {
    	ref.close();
    };

    /**
     * Display a new browser with the specified URL.
     * This method starts a new web browser activity.
     *
     * @param url           The url to load
     * @param usePhoneGap   Load url in PhoneGap webview [optional]
     */
    ChildBrowser.openExternal = function(url, usePhoneGap) {
    	return window.open(url, "_system");
    };

    if (!window.plugins) {
        window.plugins = {};
    }
    window.plugins.childBrowser = ChildBrowser;

    return ChildBrowser;
})(window.cordova || window.Cordova || window.PhoneGap);



// ----------------------



var Downloader = (function(cordova){
	function Downloader() {
	}

	Downloader.downloadFile = function(fileUrl, params, win, fail) {
		//Make params hash optional.
		if (!fail) win = params;
		cordova.exec(win, fail, "Downloader", "downloadFile", [fileUrl, params]);
	};
	
	cordova.addConstructor(function () {
	    if (cordova.addPlugin) {
	        cordova.addPlugin("downloader", Downloader);
	    } else {
	        if (!window.plugins) {
	            window.plugins = {};
	        }
	        window.plugins.downloader = Downloader;
	    }
	});	
	
	return Downloader;
})(window.cordova || window.Cordova || window.PhoneGap);



//----------------------



var Viewer = (function(cordova){
	function Viewer() {
	}

	Viewer.showFile = function(url, mimetype, success, error) {
		cordova.exec(success, error, "Viewer", "showFile", [url, mimetype]);
	};
	
	cordova.addConstructor(function () {
	    if (cordova.addPlugin) {
	        cordova.addPlugin("viewer", Viewer);
	    } else {
	        if (!window.plugins) {
	            window.plugins = {};
	        }
	        window.plugins.viewer = Viewer;
	    }
	});	
	
	return Viewer;
})(window.cordova || window.Cordova || window.PhoneGap);